//+------------------------------------------------------------------+
//|                                            MfsExpertTemplate.mq5 |
//|                        Copyright 2017, Mukachi Financial Service |
//|                                               http://www.mfs.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2017, Mukachi Financial Service"
#property link      "http://www.mfs.com"
#property version   "1.00"
// _trade
#include <MfsTrade\MfsTrade.mqh>
CTrade _trade;

// Money management
#include <MfsTrade\MfsMoneyManagement.mqh>

// Trailing stops
#include <MfsTrade\MfsTrailing.mqh>
CTrailing _trail;

// _timer
#include <MfsTrade\MfsTimer.mqh>
CTimer _timer;
CNewBar _new_bar;

// Indicators 
#include <MfsTrade\MfsSARTrader.mqh>
SARTrader *_sar;
//+------------------------------------------------------------------+
//| Input variables                                                  |
//+------------------------------------------------------------------+
input ulong Slippage=10;
input bool TradeOnNewBar=true;

sinput string MM;    // Money Management
input bool UseMoneyManagement=true;
input double RiskPercent = 2;
input double FixedVolume = 0.1;

sinput string SL;    // Stop Loss & Take Profit
input int StopLoss=200;
input int TakeProfit=1000;

sinput string TS;      // Trailing Stop
input bool UseTrailingStop=true;
input int TrailingStop=200;
input int MinimumProfit=200;
input int Step=20;

sinput string BE;      // Break Even
input bool UseBreakEven=true;
input int BreakEvenProfit=200;
input int LockProfit=50;

sinput string TI;    // Timer
input bool UseTimer = false;
input int StartHour = 0;
input int StartMinute=0;
input int EndHour=0;
input int EndMinute=0;
input bool UseLocalTime=false;

sinput string ST;    // SAR Trader
input int MAFast=21;
input int MASlow=55;
input int TrendSize=5;
input double SARStep=0.003;
input double SARMax=0.2;
//+------------------------------------------------------------------+
//| Global variables                                                 |
//+------------------------------------------------------------------+
bool buy_placed,sell_placed;
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
   _trade.deviation(Slippage);
   _sar=new SARTrader(_Symbol,_Period,SARStep,
                      SARMax,MAFast,MASlow,TrendSize);
   return(0);
  }
//+------------------------------------------------------------------+
//| Expert deitialization function                                   |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
   if(CheckPointer(_sar)==POINTER_DYNAMIC) delete(_sar);
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
// Check for new bar
   int shift=0;
   bool new_bar=true;

   if(TradeOnNewBar==true)
     {
      shift=1;
      new_bar=_new_bar.is_new_bar(_Symbol,_Period);
     }

// _timer
   bool timer_on=true;
   if(UseTimer==true)
      timer_on=_timer.daily_timer(StartHour,StartMinute,
                                  EndHour,EndMinute,
                                  UseLocalTime);

// Order placement
   if(new_bar==true && timer_on==true)
     {
      ENUM_SIGNAL signal=_sar.signal();

      ENUM_ORDER_TYPE pos=WRONG_VALUE;

      if(signal==SIGNAL_BEAR)
        {
         pos=position_type(_Symbol);
         if(pos==ORDER_TYPE_SELL) return;
         open_short();
        }
      else if(signal==SIGNAL_BULL)
        {
         pos=position_type(_Symbol);
         if(pos==ORDER_TYPE_BUY) return;
         open_long();
        }
     }

// Break even & trailing stop
   if(UseBreakEven==true && position_type(_Symbol)!=-1)
      _trail.breakeven(_Symbol,BreakEvenProfit,LockProfit);
   if(UseTrailingStop==true && position_type(_Symbol)!=-1)
      _trail.trailing_stop(_Symbol,TrailingStop,MinimumProfit,Step);
  }
//+------------------------------------------------------------------+
double lot_size()
  {
   double trade_size;
   if(UseMoneyManagement==true)
     {
      trade_size=money_management(_Symbol,FixedVolume,
                                  RiskPercent,StopLoss);
     }
   else trade_size=verify_volume(_Symbol,FixedVolume);
   return(trade_size);
  }
//+------------------------------------------------------------------+
void open_long()
  {
   buy_placed=_trade.buy(_Symbol,lot_size());

   if(buy_placed==true)
     {
      do(Sleep(100));
      while(PositionSelect(_Symbol)==false);
      double open_price=position_open_price(_Symbol);

      double buy_stop=buy_stop_loss(_Symbol,StopLoss,open_price);
      if(buy_stop>0) adjust_below_stop_level(_Symbol,buy_stop);

      double buy_profit=buy_take_profit(_Symbol,TakeProfit,open_price);
      if(buy_profit>0) adjust_above_stop_level(_Symbol,buy_profit);

      if(buy_stop>0 || buy_profit>0)
         _trade.modify_market(_Symbol,buy_stop,buy_profit);
     }
  }
//+------------------------------------------------------------------+
void open_short()
  {
   sell_placed=_trade.sell(_Symbol,lot_size());

   if(sell_placed==true)
     {
      do(Sleep(100));
      while(PositionSelect(_Symbol)==false);
      double open_price=position_open_price(_Symbol);

      double sell_stop=sell_stop_loss(_Symbol,StopLoss,open_price);
      if(sell_stop>0) sell_stop=adjust_above_stop_level(_Symbol,sell_stop);

      double sell_profit=sell_take_profit(_Symbol,TakeProfit,open_price);
      if(sell_profit>0) sell_profit=adjust_below_stop_level(_Symbol,sell_profit);

      if(sell_stop>0 || sell_profit>0)
         _trade.modify_market(_Symbol,sell_stop,sell_profit);
     }
  }
//+------------------------------------------------------------------+
